# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import IamErrorResponse
from ..models import ModelCreateGoalRequest
from ..models import ModelGetGoalsResponse
from ..models import ModelGetSlotsResponse
from ..models import ModelGoalResponse
from ..models import ModelMoveGoalToSlotRequest
from ..models import ModelUpdateGoalRequest
from ..models import ResponseError

from ..operations.goal_configuration import AdminCreateGoal
from ..operations.goal_configuration import AdminDeleteGoal
from ..operations.goal_configuration import AdminGetChallengeSlots
from ..operations.goal_configuration import AdminGetGoal
from ..operations.goal_configuration import AdminGetGoals
from ..operations.goal_configuration import AdminGetGoalsSortByEnum
from ..operations.goal_configuration import AdminMoveGoalToSlot
from ..operations.goal_configuration import AdminUpdateGoals


@same_doc_as(AdminCreateGoal)
def admin_create_goal(
    body: ModelCreateGoalRequest,
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create New Goal (adminCreateGoal)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [CREATE]
    Request body:
    - code: unique within a challenge.
    - name: name of the goal.
    - description: text describing the goal (optional).
    - schedule: a time range that indicated the availability of a goal within a timeframe. Used in FIXED assignment rule, this will be required in that case.
    - requirementGroups: contains array of objects consisting of operator and predicates attribute. When requirements evaluation happened, each item's predicates will be evaluated first, by operating the predicates with given operator.
    After all items evaluated, the results of the evaluation on each item then will be logically evaluated using OR operator.
    - operator: logical operator used to validate the completion of a goal. a goal is considered complete once complete predicates operated with operator result in true.
    - predicates: list of progression parameters to be tracked.
    - parameterType: the type of parameter for challenge to be progressed with. the available options are:
    - STATISTIC: progress by user statistic item value.
    - STATISTIC_CYCLE: progress user statistic cycle item value. statCycleId must be included.
    - ACHIEVEMENT: progress by user achievement.
    - USERACCOUNT: progress by user account event.
    - ENTITLEMENT: progress by user's item entitlement ownership. Supported item types are APP, CODE, INGAMEITEM, LOOTBOX, MEDIA, and OPTIONBOX.
    - parameterName: the name of the parameter for challenge to be progressed with.
    - STATISTIC: refers to stat code.
    - STATISTIC_CYCLE: refers to stat code with statCycleId must be included.
    - ACHIEVEMENT: refers to achievement code.
    - USERACCOUNT: refers to a specific event related to user accounts. The current possible events include: userAccountCreated, gameUserAccountCreated, userAccountVerified, userAccountLinked, userAccountUpgraded, thirdPartyAccountCreated, userLoggedIn, userThirdPartyLoggedIn and fullAccount.
    - For the **userAccountLinked** event, **parameterName** will utilize **platformId** rather than the event name. This will be validated against the list of active third-party accounts configuration within the namespace, such as *psn*, *steam*, or *device*.
    - ENTITLEMENT: ecommerce store's item SKU.
    - matcher: the comparison operator used to compare the current value of a parameter and targetValue to validate the completion of a predicate. Possible values are EQUAL, LESS_THAN, GREATER_THAN, LESS_THAN_EQUAL, and GREATER_THAN_EQUAL.
    - targetValue: the target number to be reached by the parameter.
    - statCycleId: specify the statCycleId used to track statistic value in a cycle with STATISTIC_CYCLE parameterType (optional).
    - rewards: list of rewards that will be claimable once a goal is complete.
    - tags: goal's labels.
    - isActive: when goal is in a schedule, isActive determine whether goal is active to progress or not.
    Goal describe set of requirements that need to be fulfilled by players in order to complete it and describe what is the rewards given to player when they complete the goal.
    The requirement will have target value and a operator that will evaluate that against an observable playerâs attribute (e.g. statistic, entitlement). Goal belongs to a challenge.
    Supported item type for ENTITLEMENT reward type: APP, BUNDLE, CODE, COINS, EXTENSION, INGAMEITEM, LOOTBOX, MEDIA, OPTIONBOX.
    Number of goals per challenge is **limited to 100 goals**.

    **Important**
    Add a new goal in a TIED challenge may affect users' data, please do not do this unless it's necessary. Then new goal will take effect in the next evaluation

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals

        method: POST

        tags: ["Goal Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelCreateGoalRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelGoalResponse (Created)

        400: Bad Request - IamErrorResponse (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        409: Conflict - ResponseError (99002: duplicate key error: {{message}})

        422: Unprocessable Entity - IamErrorResponse (99004: unprocessable entity: {{message}})

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateGoal.create(
        body=body,
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateGoal)
async def admin_create_goal_async(
    body: ModelCreateGoalRequest,
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create New Goal (adminCreateGoal)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [CREATE]
    Request body:
    - code: unique within a challenge.
    - name: name of the goal.
    - description: text describing the goal (optional).
    - schedule: a time range that indicated the availability of a goal within a timeframe. Used in FIXED assignment rule, this will be required in that case.
    - requirementGroups: contains array of objects consisting of operator and predicates attribute. When requirements evaluation happened, each item's predicates will be evaluated first, by operating the predicates with given operator.
    After all items evaluated, the results of the evaluation on each item then will be logically evaluated using OR operator.
    - operator: logical operator used to validate the completion of a goal. a goal is considered complete once complete predicates operated with operator result in true.
    - predicates: list of progression parameters to be tracked.
    - parameterType: the type of parameter for challenge to be progressed with. the available options are:
    - STATISTIC: progress by user statistic item value.
    - STATISTIC_CYCLE: progress user statistic cycle item value. statCycleId must be included.
    - ACHIEVEMENT: progress by user achievement.
    - USERACCOUNT: progress by user account event.
    - ENTITLEMENT: progress by user's item entitlement ownership. Supported item types are APP, CODE, INGAMEITEM, LOOTBOX, MEDIA, and OPTIONBOX.
    - parameterName: the name of the parameter for challenge to be progressed with.
    - STATISTIC: refers to stat code.
    - STATISTIC_CYCLE: refers to stat code with statCycleId must be included.
    - ACHIEVEMENT: refers to achievement code.
    - USERACCOUNT: refers to a specific event related to user accounts. The current possible events include: userAccountCreated, gameUserAccountCreated, userAccountVerified, userAccountLinked, userAccountUpgraded, thirdPartyAccountCreated, userLoggedIn, userThirdPartyLoggedIn and fullAccount.
    - For the **userAccountLinked** event, **parameterName** will utilize **platformId** rather than the event name. This will be validated against the list of active third-party accounts configuration within the namespace, such as *psn*, *steam*, or *device*.
    - ENTITLEMENT: ecommerce store's item SKU.
    - matcher: the comparison operator used to compare the current value of a parameter and targetValue to validate the completion of a predicate. Possible values are EQUAL, LESS_THAN, GREATER_THAN, LESS_THAN_EQUAL, and GREATER_THAN_EQUAL.
    - targetValue: the target number to be reached by the parameter.
    - statCycleId: specify the statCycleId used to track statistic value in a cycle with STATISTIC_CYCLE parameterType (optional).
    - rewards: list of rewards that will be claimable once a goal is complete.
    - tags: goal's labels.
    - isActive: when goal is in a schedule, isActive determine whether goal is active to progress or not.
    Goal describe set of requirements that need to be fulfilled by players in order to complete it and describe what is the rewards given to player when they complete the goal.
    The requirement will have target value and a operator that will evaluate that against an observable playerâs attribute (e.g. statistic, entitlement). Goal belongs to a challenge.
    Supported item type for ENTITLEMENT reward type: APP, BUNDLE, CODE, COINS, EXTENSION, INGAMEITEM, LOOTBOX, MEDIA, OPTIONBOX.
    Number of goals per challenge is **limited to 100 goals**.

    **Important**
    Add a new goal in a TIED challenge may affect users' data, please do not do this unless it's necessary. Then new goal will take effect in the next evaluation

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals

        method: POST

        tags: ["Goal Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelCreateGoalRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelGoalResponse (Created)

        400: Bad Request - IamErrorResponse (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        409: Conflict - ResponseError (99002: duplicate key error: {{message}})

        422: Unprocessable Entity - IamErrorResponse (99004: unprocessable entity: {{message}})

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateGoal.create(
        body=body,
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminDeleteGoal)
def admin_delete_goal(
    challenge_code: str,
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Goal (adminDeleteGoal)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [DELETE]

    **Important**
    Delete a goal in a TIED challenge may affect/delete users' data, please do not delete a TIED goal unless it's necessary. User data will be deleted in the next evaluation

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals/{code}

        method: DELETE

        tags: ["Goal Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteGoal.create(
        challenge_code=challenge_code,
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteGoal)
async def admin_delete_goal_async(
    challenge_code: str,
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Goal (adminDeleteGoal)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [DELETE]

    **Important**
    Delete a goal in a TIED challenge may affect/delete users' data, please do not delete a TIED goal unless it's necessary. User data will be deleted in the next evaluation

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals/{code}

        method: DELETE

        tags: ["Goal Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteGoal.create(
        challenge_code=challenge_code,
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetChallengeSlots)
def admin_get_challenge_slots(
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Challenge Slots (adminGetChallengeSlots)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]
    Returns the slot configuration for a FIXED or RANDOMIZED (non-per-rotation) assignment challenge.
    For repeating challenges (repeatAfter set), the response contains two separate fields:
    - templateSlots: the authoritative future configuration from TemplateSlots (goals only, no timing).
    - currentRound: the actual running state of the current (or nearest) round (goals + startTime/endTime from schedule documents).
    These two may differ when a slot move was deferred because the target slot was active.
    For non-repeating challenges, only currentRound is returned (templateSlots is absent).

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/slots

        method: GET

        tags: ["Goal Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelGetSlotsResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        422: Unprocessable Entity - ResponseError (99004: unprocessable entity: {{message}})

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetChallengeSlots.create(
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetChallengeSlots)
async def admin_get_challenge_slots_async(
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Challenge Slots (adminGetChallengeSlots)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]
    Returns the slot configuration for a FIXED or RANDOMIZED (non-per-rotation) assignment challenge.
    For repeating challenges (repeatAfter set), the response contains two separate fields:
    - templateSlots: the authoritative future configuration from TemplateSlots (goals only, no timing).
    - currentRound: the actual running state of the current (or nearest) round (goals + startTime/endTime from schedule documents).
    These two may differ when a slot move was deferred because the target slot was active.
    For non-repeating challenges, only currentRound is returned (templateSlots is absent).

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/slots

        method: GET

        tags: ["Goal Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelGetSlotsResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        422: Unprocessable Entity - ResponseError (99004: unprocessable entity: {{message}})

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetChallengeSlots.create(
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetGoal)
def admin_get_goal(
    challenge_code: str,
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Goal (adminGetGoal)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals/{code}

        method: GET

        tags: ["Goal Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelGoalResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetGoal.create(
        challenge_code=challenge_code,
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetGoal)
async def admin_get_goal_async(
    challenge_code: str,
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Goal (adminGetGoal)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals/{code}

        method: GET

        tags: ["Goal Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelGoalResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetGoal.create(
        challenge_code=challenge_code,
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetGoals)
def admin_get_goals(
    challenge_code: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, AdminGetGoalsSortByEnum]] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List Goals of a Challenge (adminGetGoals)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals

        method: GET

        tags: ["Goal Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelGetGoalsResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetGoals.create(
        challenge_code=challenge_code,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        tags=tags,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetGoals)
async def admin_get_goals_async(
    challenge_code: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, AdminGetGoalsSortByEnum]] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List Goals of a Challenge (adminGetGoals)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals

        method: GET

        tags: ["Goal Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelGetGoalsResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetGoals.create(
        challenge_code=challenge_code,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        tags=tags,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminMoveGoalToSlot)
def admin_move_goal_to_slot(
    body: ModelMoveGoalToSlotRequest,
    challenge_code: str,
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Move Goal to Another Slot (adminMoveGoalToSlot)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    Moves a goal to a target slot in a FIXED or non-per-rotation RANDOMIZED challenge with repeatAfter set.
    slotIndex is 0-based and must be in [0, repeatAfter-1].
    Pass slotIndex = -1 to remove the goal from all slots; it will no longer appear in future rounds.
    TemplateSlots and all non-finished schedule documents are updated immediately.
    The currently active schedule document for the affected slot is skipped to avoid
    disrupting in-progress user progressions; the change takes effect from the next round.
    Finished (past) schedule documents are never modified.

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals/{code}/slots

        method: PUT

        tags: ["Goal Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelMoveGoalToSlotRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (goal moved successfully)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        422: Unprocessable Entity - ResponseError (99004: unprocessable entity: {{message}})

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminMoveGoalToSlot.create(
        body=body,
        challenge_code=challenge_code,
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminMoveGoalToSlot)
async def admin_move_goal_to_slot_async(
    body: ModelMoveGoalToSlotRequest,
    challenge_code: str,
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Move Goal to Another Slot (adminMoveGoalToSlot)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    Moves a goal to a target slot in a FIXED or non-per-rotation RANDOMIZED challenge with repeatAfter set.
    slotIndex is 0-based and must be in [0, repeatAfter-1].
    Pass slotIndex = -1 to remove the goal from all slots; it will no longer appear in future rounds.
    TemplateSlots and all non-finished schedule documents are updated immediately.
    The currently active schedule document for the affected slot is skipped to avoid
    disrupting in-progress user progressions; the change takes effect from the next round.
    Finished (past) schedule documents are never modified.

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals/{code}/slots

        method: PUT

        tags: ["Goal Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelMoveGoalToSlotRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (goal moved successfully)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        422: Unprocessable Entity - ResponseError (99004: unprocessable entity: {{message}})

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminMoveGoalToSlot.create(
        body=body,
        challenge_code=challenge_code,
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateGoals)
def admin_update_goals(
    body: ModelUpdateGoalRequest,
    challenge_code: str,
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Goal (adminUpdateGoals)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    ### Request body:
    - name: name of the goal.
    - description: text describing the goal (optional).
    - schedule: a time range that indicated the availability of a goal within a timeframe. Used in FIXED assignment rule, this will be required in that case.
    - requirementGroups: contains array of objects consisting of operator and predicates attribute. When requirements evaluation happened, each item's predicates will be evaluated first, by operating the predicates with given operator. After all items evaluated, the results of the evaluation on each item then will be logically evaluated using OR operator.
    - operator: logical operator used to validate the completion of a goal. a goal is considered complete once complete predicates operated with operator result in true.
    - predicates: list of progression parameters to be tracked.
    - parameterType: the type of parameter for challenge to be progressed with. the available options are:
    - STATISTIC: progress by user statistic item value.
    - STATISTIC_CYCLE: progress user statistic cycle item value. statCycleId must be included.
    - ACHIEVEMENT: progress by user achievement.
    - USERACCOUNT: progress by user account event.
    - ENTITLEMENT: progress by user's item entitlement ownership. Supported item types are APP, CODE, INGAMEITEM, LOOTBOX, MEDIA, and OPTIONBOX.
    - parameterName: the name of the parameter for challenge to be progressed with.
    - STATISTIC: refers to stat code.
    - STATISTIC_CYCLE: refers to stat code with statCycleId must be included.
    - ACHIEVEMENT: refers to achievement code.
    - USERACCOUNT: one of the user account event. Current possible values are (userAccountCreated, gameUserAccountCreated, userAccountVerified, userAccountLinked, userAccountUpgraded,thirdPartyAccountCreated).
    - ENTITLEMENT: ecommerce store's item SKU.
    - matcher: the comparison operator used to compare the current value of a parameter and targetValue to validate the completion of a predicate. Possible values are EQUAL, LESS_THAN, GREATER_THAN, LESS_THAN_EQUAL, and GREATER_THAN_EQUAL.
    - targetValue: the target number to be reached by the parameter.
    - statCycleId: specify the statCycleId used to track statistic value in a cycle with STATISTIC_CYCLE parameterType (optional).
    - rewards: list of rewards that will be claimable once a goal is complete.
    - tags: goal's labels.
    - isActive: when goal is in a schedule, isActive determine whether goal is active to progress or not (optional).
    Goal describe set of requirements that need to be fulfilled by players in order to complete it and describe what is the rewards given to player when they complete the goal.
    The requirement will have target value and a operator that will evaluate that against an observable playerâs attribute (e.g. statistic, entitlement). Goal belongs to a challenge.
    Supported item type for ENTITLEMENT reward type: APP, BUNDLE, CODE, COINS, EXTENSION, INGAMEITEM, LOOTBOX, MEDIA, OPTIONBOX

    **Important**
    - Update a goal in a TIED challenge may affect users' data, please do not update a TIED goal unless it's necessary. The updated goal will take effect in the next evaluation.
    - When updating an existing predicate, ensure to put the predicate id in the request, otherwise a new predicate will be created with a new id

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals/{code}

        method: PUT

        tags: ["Goal Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUpdateGoalRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelGoalResponse (OK)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        404: Not Found - ResponseError (Not Found)

        422: Unprocessable Entity - ResponseError (99004: unprocessable entity: {{message}})

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateGoals.create(
        body=body,
        challenge_code=challenge_code,
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateGoals)
async def admin_update_goals_async(
    body: ModelUpdateGoalRequest,
    challenge_code: str,
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Goal (adminUpdateGoals)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    ### Request body:
    - name: name of the goal.
    - description: text describing the goal (optional).
    - schedule: a time range that indicated the availability of a goal within a timeframe. Used in FIXED assignment rule, this will be required in that case.
    - requirementGroups: contains array of objects consisting of operator and predicates attribute. When requirements evaluation happened, each item's predicates will be evaluated first, by operating the predicates with given operator. After all items evaluated, the results of the evaluation on each item then will be logically evaluated using OR operator.
    - operator: logical operator used to validate the completion of a goal. a goal is considered complete once complete predicates operated with operator result in true.
    - predicates: list of progression parameters to be tracked.
    - parameterType: the type of parameter for challenge to be progressed with. the available options are:
    - STATISTIC: progress by user statistic item value.
    - STATISTIC_CYCLE: progress user statistic cycle item value. statCycleId must be included.
    - ACHIEVEMENT: progress by user achievement.
    - USERACCOUNT: progress by user account event.
    - ENTITLEMENT: progress by user's item entitlement ownership. Supported item types are APP, CODE, INGAMEITEM, LOOTBOX, MEDIA, and OPTIONBOX.
    - parameterName: the name of the parameter for challenge to be progressed with.
    - STATISTIC: refers to stat code.
    - STATISTIC_CYCLE: refers to stat code with statCycleId must be included.
    - ACHIEVEMENT: refers to achievement code.
    - USERACCOUNT: one of the user account event. Current possible values are (userAccountCreated, gameUserAccountCreated, userAccountVerified, userAccountLinked, userAccountUpgraded,thirdPartyAccountCreated).
    - ENTITLEMENT: ecommerce store's item SKU.
    - matcher: the comparison operator used to compare the current value of a parameter and targetValue to validate the completion of a predicate. Possible values are EQUAL, LESS_THAN, GREATER_THAN, LESS_THAN_EQUAL, and GREATER_THAN_EQUAL.
    - targetValue: the target number to be reached by the parameter.
    - statCycleId: specify the statCycleId used to track statistic value in a cycle with STATISTIC_CYCLE parameterType (optional).
    - rewards: list of rewards that will be claimable once a goal is complete.
    - tags: goal's labels.
    - isActive: when goal is in a schedule, isActive determine whether goal is active to progress or not (optional).
    Goal describe set of requirements that need to be fulfilled by players in order to complete it and describe what is the rewards given to player when they complete the goal.
    The requirement will have target value and a operator that will evaluate that against an observable playerâs attribute (e.g. statistic, entitlement). Goal belongs to a challenge.
    Supported item type for ENTITLEMENT reward type: APP, BUNDLE, CODE, COINS, EXTENSION, INGAMEITEM, LOOTBOX, MEDIA, OPTIONBOX

    **Important**
    - Update a goal in a TIED challenge may affect users' data, please do not update a TIED goal unless it's necessary. The updated goal will take effect in the next evaluation.
    - When updating an existing predicate, ensure to put the predicate id in the request, otherwise a new predicate will be created with a new id

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/goals/{code}

        method: PUT

        tags: ["Goal Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUpdateGoalRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelGoalResponse (OK)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        404: Not Found - ResponseError (Not Found)

        422: Unprocessable Entity - ResponseError (99004: unprocessable entity: {{message}})

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateGoals.create(
        body=body,
        challenge_code=challenge_code,
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApimodelsRefreshTicketResponse
from ..models import ResponseError

from ..operations.ticket_v1 import CancelTicket
from ..operations.ticket_v1 import RefreshTicket


@same_doc_as(CancelTicket)
def cancel_ticket(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Cancel ticket (cancelTicket)

    Cancel ticket. This endpoint requires ticket to be placed in the authorization header with this format: "Bearer ticketID"

    Properties:
        url: /login-queue/v1/namespaces/{namespace}/ticket

        method: DELETE

        tags: ["Ticket V1"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CancelTicket.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CancelTicket)
async def cancel_ticket_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Cancel ticket (cancelTicket)

    Cancel ticket. This endpoint requires ticket to be placed in the authorization header with this format: "Bearer ticketID"

    Properties:
        url: /login-queue/v1/namespaces/{namespace}/ticket

        method: DELETE

        tags: ["Ticket V1"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CancelTicket.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RefreshTicket)
def refresh_ticket(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Refresh ticket (refreshTicket)

    Refresh ticket. This endpoint requires ticket to be placed in the authorization header with this format: "Bearer ticketID". When the ticketID is empty or not found the response status will be 401

    Properties:
        url: /login-queue/v1/namespaces/{namespace}/ticket

        method: GET

        tags: ["Ticket V1"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsRefreshTicketResponse (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RefreshTicket.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RefreshTicket)
async def refresh_ticket_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Refresh ticket (refreshTicket)

    Refresh ticket. This endpoint requires ticket to be placed in the authorization header with this format: "Bearer ticketID". When the ticketID is empty or not found the response status will be 401

    Properties:
        url: /login-queue/v1/namespaces/{namespace}/ticket

        method: GET

        tags: ["Ticket V1"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsRefreshTicketResponse (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RefreshTicket.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Fleet Commander

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApiImageStorage(Model):
    """Api image storage (api.ImageStorage)

    Properties:
        current_marked_for_deletion_bytes: (currentMarkedForDeletionBytes) REQUIRED int

        current_usage_bytes: (currentUsageBytes) REQUIRED int

        quota_bytes: (quotaBytes) REQUIRED int
    """

    # region fields

    current_marked_for_deletion_bytes: int  # REQUIRED
    current_usage_bytes: int  # REQUIRED
    quota_bytes: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_current_marked_for_deletion_bytes(self, value: int) -> ApiImageStorage:
        self.current_marked_for_deletion_bytes = value
        return self

    def with_current_usage_bytes(self, value: int) -> ApiImageStorage:
        self.current_usage_bytes = value
        return self

    def with_quota_bytes(self, value: int) -> ApiImageStorage:
        self.quota_bytes = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "current_marked_for_deletion_bytes"):
            result["currentMarkedForDeletionBytes"] = int(
                self.current_marked_for_deletion_bytes
            )
        elif include_empty:
            result["currentMarkedForDeletionBytes"] = 0
        if hasattr(self, "current_usage_bytes"):
            result["currentUsageBytes"] = int(self.current_usage_bytes)
        elif include_empty:
            result["currentUsageBytes"] = 0
        if hasattr(self, "quota_bytes"):
            result["quotaBytes"] = int(self.quota_bytes)
        elif include_empty:
            result["quotaBytes"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        current_marked_for_deletion_bytes: int,
        current_usage_bytes: int,
        quota_bytes: int,
        **kwargs,
    ) -> ApiImageStorage:
        instance = cls()
        instance.current_marked_for_deletion_bytes = current_marked_for_deletion_bytes
        instance.current_usage_bytes = current_usage_bytes
        instance.quota_bytes = quota_bytes
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApiImageStorage:
        instance = cls()
        if not dict_:
            return instance
        if (
            "currentMarkedForDeletionBytes" in dict_
            and dict_["currentMarkedForDeletionBytes"] is not None
        ):
            instance.current_marked_for_deletion_bytes = int(
                dict_["currentMarkedForDeletionBytes"]
            )
        elif include_empty:
            instance.current_marked_for_deletion_bytes = 0
        if "currentUsageBytes" in dict_ and dict_["currentUsageBytes"] is not None:
            instance.current_usage_bytes = int(dict_["currentUsageBytes"])
        elif include_empty:
            instance.current_usage_bytes = 0
        if "quotaBytes" in dict_ and dict_["quotaBytes"] is not None:
            instance.quota_bytes = int(dict_["quotaBytes"])
        elif include_empty:
            instance.quota_bytes = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiImageStorage]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiImageStorage]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ApiImageStorage, List[ApiImageStorage], Dict[Any, ApiImageStorage]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "currentMarkedForDeletionBytes": "current_marked_for_deletion_bytes",
            "currentUsageBytes": "current_usage_bytes",
            "quotaBytes": "quota_bytes",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "currentMarkedForDeletionBytes": True,
            "currentUsageBytes": True,
            "quotaBytes": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ItemReference(Model):
    """Item reference (ItemReference)

    Properties:
        code_redemption_id: (codeRedemptionId) OPTIONAL str

        condition_name: (conditionName) OPTIONAL str

        dlc_id: (dlcId) OPTIONAL str

        item_id: (itemId) OPTIONAL str

        namespace: (namespace) OPTIONAL str

        platform: (platform) OPTIONAL str

        platform_product_id: (platformProductId) OPTIONAL str

        reference_id: (referenceId) OPTIONAL str

        reward_code: (rewardCode) OPTIONAL str

        store_id: (storeId) OPTIONAL str
    """

    # region fields

    code_redemption_id: str  # OPTIONAL
    condition_name: str  # OPTIONAL
    dlc_id: str  # OPTIONAL
    item_id: str  # OPTIONAL
    namespace: str  # OPTIONAL
    platform: str  # OPTIONAL
    platform_product_id: str  # OPTIONAL
    reference_id: str  # OPTIONAL
    reward_code: str  # OPTIONAL
    store_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_code_redemption_id(self, value: str) -> ItemReference:
        self.code_redemption_id = value
        return self

    def with_condition_name(self, value: str) -> ItemReference:
        self.condition_name = value
        return self

    def with_dlc_id(self, value: str) -> ItemReference:
        self.dlc_id = value
        return self

    def with_item_id(self, value: str) -> ItemReference:
        self.item_id = value
        return self

    def with_namespace(self, value: str) -> ItemReference:
        self.namespace = value
        return self

    def with_platform(self, value: str) -> ItemReference:
        self.platform = value
        return self

    def with_platform_product_id(self, value: str) -> ItemReference:
        self.platform_product_id = value
        return self

    def with_reference_id(self, value: str) -> ItemReference:
        self.reference_id = value
        return self

    def with_reward_code(self, value: str) -> ItemReference:
        self.reward_code = value
        return self

    def with_store_id(self, value: str) -> ItemReference:
        self.store_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "code_redemption_id"):
            result["codeRedemptionId"] = str(self.code_redemption_id)
        elif include_empty:
            result["codeRedemptionId"] = ""
        if hasattr(self, "condition_name"):
            result["conditionName"] = str(self.condition_name)
        elif include_empty:
            result["conditionName"] = ""
        if hasattr(self, "dlc_id"):
            result["dlcId"] = str(self.dlc_id)
        elif include_empty:
            result["dlcId"] = ""
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "platform"):
            result["platform"] = str(self.platform)
        elif include_empty:
            result["platform"] = ""
        if hasattr(self, "platform_product_id"):
            result["platformProductId"] = str(self.platform_product_id)
        elif include_empty:
            result["platformProductId"] = ""
        if hasattr(self, "reference_id"):
            result["referenceId"] = str(self.reference_id)
        elif include_empty:
            result["referenceId"] = ""
        if hasattr(self, "reward_code"):
            result["rewardCode"] = str(self.reward_code)
        elif include_empty:
            result["rewardCode"] = ""
        if hasattr(self, "store_id"):
            result["storeId"] = str(self.store_id)
        elif include_empty:
            result["storeId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        code_redemption_id: Optional[str] = None,
        condition_name: Optional[str] = None,
        dlc_id: Optional[str] = None,
        item_id: Optional[str] = None,
        namespace: Optional[str] = None,
        platform: Optional[str] = None,
        platform_product_id: Optional[str] = None,
        reference_id: Optional[str] = None,
        reward_code: Optional[str] = None,
        store_id: Optional[str] = None,
        **kwargs,
    ) -> ItemReference:
        instance = cls()
        if code_redemption_id is not None:
            instance.code_redemption_id = code_redemption_id
        if condition_name is not None:
            instance.condition_name = condition_name
        if dlc_id is not None:
            instance.dlc_id = dlc_id
        if item_id is not None:
            instance.item_id = item_id
        if namespace is not None:
            instance.namespace = namespace
        if platform is not None:
            instance.platform = platform
        if platform_product_id is not None:
            instance.platform_product_id = platform_product_id
        if reference_id is not None:
            instance.reference_id = reference_id
        if reward_code is not None:
            instance.reward_code = reward_code
        if store_id is not None:
            instance.store_id = store_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ItemReference:
        instance = cls()
        if not dict_:
            return instance
        if "codeRedemptionId" in dict_ and dict_["codeRedemptionId"] is not None:
            instance.code_redemption_id = str(dict_["codeRedemptionId"])
        elif include_empty:
            instance.code_redemption_id = ""
        if "conditionName" in dict_ and dict_["conditionName"] is not None:
            instance.condition_name = str(dict_["conditionName"])
        elif include_empty:
            instance.condition_name = ""
        if "dlcId" in dict_ and dict_["dlcId"] is not None:
            instance.dlc_id = str(dict_["dlcId"])
        elif include_empty:
            instance.dlc_id = ""
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "platform" in dict_ and dict_["platform"] is not None:
            instance.platform = str(dict_["platform"])
        elif include_empty:
            instance.platform = ""
        if "platformProductId" in dict_ and dict_["platformProductId"] is not None:
            instance.platform_product_id = str(dict_["platformProductId"])
        elif include_empty:
            instance.platform_product_id = ""
        if "referenceId" in dict_ and dict_["referenceId"] is not None:
            instance.reference_id = str(dict_["referenceId"])
        elif include_empty:
            instance.reference_id = ""
        if "rewardCode" in dict_ and dict_["rewardCode"] is not None:
            instance.reward_code = str(dict_["rewardCode"])
        elif include_empty:
            instance.reward_code = ""
        if "storeId" in dict_ and dict_["storeId"] is not None:
            instance.store_id = str(dict_["storeId"])
        elif include_empty:
            instance.store_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ItemReference]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ItemReference]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ItemReference, List[ItemReference], Dict[Any, ItemReference]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "codeRedemptionId": "code_redemption_id",
            "conditionName": "condition_name",
            "dlcId": "dlc_id",
            "itemId": "item_id",
            "namespace": "namespace",
            "platform": "platform",
            "platformProductId": "platform_product_id",
            "referenceId": "reference_id",
            "rewardCode": "reward_code",
            "storeId": "store_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "codeRedemptionId": False,
            "conditionName": False,
            "dlcId": False,
            "itemId": False,
            "namespace": False,
            "platform": False,
            "platformProductId": False,
            "referenceId": False,
            "rewardCode": False,
            "storeId": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApimodelsXRayBulkTicketObservabilityRequest
from ..models import ApimodelsXRayBulkTicketObservabilityResponse
from ..models import ApimodelsXRayTicketObservabilityRequest
from ..models import ApimodelsXRayTicketObservabilityResponse
from ..models import ResponseError

from ..operations.x_ray import CreateXrayBulkTicketObservability
from ..operations.x_ray import CreateXrayTicketObservability


@same_doc_as(CreateXrayBulkTicketObservability)
def create_xray_bulk_ticket_observability(
    body: ApimodelsXRayBulkTicketObservabilityRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create bulk ticket observability request (createXrayBulkTicketObservability)

    Create bulk ticket observability request
    Request body details (all attributes are optional):
    Timestamp : timestamp when calling this endpoint
    Action : support one of the following value:
    1. "matchFound"
    2. "matchNotFound"
    3. "flexed"
    PartyID : ticket Party ID
    MatchID : match ID will be filled only when match found
    Namespace : ticket current namespace
    GameMode : ticket current matchpool
    ActiveAllianceRule : current active alliance ruleset
    ActiveMatchingRule : current active matching ruleset
    Function : name of the function that called the endpoint
    Iteration : total iteration before match found
    TimeToMatchSec : time to match (in seconds) will be filled only when match found
    UnmatchReason : reason when unable to find match
    RemainingTickets : remaining ticket when unable to find match
    RemainingPlayersPerTicket : remaining players when unable to find match
    UnbackfillReason : reason when unable to backfill
    IsBackfillMatch : flag to distinguish between new match and backfill match
    IsRuleSetFlexed : flag if ruleset is getting flexed
    TickID : tick id for the matchmaking tick
    SessionTickID : session tick id for differentiate session when doing matches

    Properties:
        url: /sessionhistory/v2/admin/namespaces/{namespace}/xray/tickets/bulk

        method: POST

        tags: ["XRay"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsXRayBulkTicketObservabilityRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsXRayBulkTicketObservabilityResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateXrayBulkTicketObservability.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateXrayBulkTicketObservability)
async def create_xray_bulk_ticket_observability_async(
    body: ApimodelsXRayBulkTicketObservabilityRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create bulk ticket observability request (createXrayBulkTicketObservability)

    Create bulk ticket observability request
    Request body details (all attributes are optional):
    Timestamp : timestamp when calling this endpoint
    Action : support one of the following value:
    1. "matchFound"
    2. "matchNotFound"
    3. "flexed"
    PartyID : ticket Party ID
    MatchID : match ID will be filled only when match found
    Namespace : ticket current namespace
    GameMode : ticket current matchpool
    ActiveAllianceRule : current active alliance ruleset
    ActiveMatchingRule : current active matching ruleset
    Function : name of the function that called the endpoint
    Iteration : total iteration before match found
    TimeToMatchSec : time to match (in seconds) will be filled only when match found
    UnmatchReason : reason when unable to find match
    RemainingTickets : remaining ticket when unable to find match
    RemainingPlayersPerTicket : remaining players when unable to find match
    UnbackfillReason : reason when unable to backfill
    IsBackfillMatch : flag to distinguish between new match and backfill match
    IsRuleSetFlexed : flag if ruleset is getting flexed
    TickID : tick id for the matchmaking tick
    SessionTickID : session tick id for differentiate session when doing matches

    Properties:
        url: /sessionhistory/v2/admin/namespaces/{namespace}/xray/tickets/bulk

        method: POST

        tags: ["XRay"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsXRayBulkTicketObservabilityRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsXRayBulkTicketObservabilityResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateXrayBulkTicketObservability.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateXrayTicketObservability)
def create_xray_ticket_observability(
    body: ApimodelsXRayTicketObservabilityRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create ticket observability request (createXrayTicketObservability)

    Create ticket observability request
    Request body details (all attributes are optional):
    Timestamp : timestamp when calling this endpoint
    Action : support one of the following value:
    1. "matchFound"
    2. "matchNotFound"
    3. "flexed"
    TicketID : ticket ID
    MatchID : match ID will be filled only when match found
    Namespace : ticket current namespace
    GameMode : ticket current matchpool
    ActiveAllianceRule : current active alliance ruleset
    ActiveMatchingRule : current active matching ruleset
    Function : name of the function that called the endpoint
    Iteration : total iteration before match found
    TimeToMatchSec : time to match (in seconds) will be filled only when match found
    UnmatchReason : reason when unable to find match
    RemainingTickets : remaining ticket when unable to find match
    RemainingPlayersPerTicket : remaining players when unable to find match
    UnbackfillReason : reason when unable to backfill
    IsBackfillMatch : flag to distinguish between new match and backfill match
    IsRuleSetFlexed : flag if ruleset is getting flexed
    TickID : tick id for the matchmaking tick
    SessionTickID : session tick id for differentiate session when doing matches

    Properties:
        url: /sessionhistory/v2/admin/namespaces/{namespace}/xray/tickets

        method: POST

        tags: ["XRay"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsXRayTicketObservabilityRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsXRayTicketObservabilityResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateXrayTicketObservability.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateXrayTicketObservability)
async def create_xray_ticket_observability_async(
    body: ApimodelsXRayTicketObservabilityRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create ticket observability request (createXrayTicketObservability)

    Create ticket observability request
    Request body details (all attributes are optional):
    Timestamp : timestamp when calling this endpoint
    Action : support one of the following value:
    1. "matchFound"
    2. "matchNotFound"
    3. "flexed"
    TicketID : ticket ID
    MatchID : match ID will be filled only when match found
    Namespace : ticket current namespace
    GameMode : ticket current matchpool
    ActiveAllianceRule : current active alliance ruleset
    ActiveMatchingRule : current active matching ruleset
    Function : name of the function that called the endpoint
    Iteration : total iteration before match found
    TimeToMatchSec : time to match (in seconds) will be filled only when match found
    UnmatchReason : reason when unable to find match
    RemainingTickets : remaining ticket when unable to find match
    RemainingPlayersPerTicket : remaining players when unable to find match
    UnbackfillReason : reason when unable to backfill
    IsBackfillMatch : flag to distinguish between new match and backfill match
    IsRuleSetFlexed : flag if ruleset is getting flexed
    TickID : tick id for the matchmaking tick
    SessionTickID : session tick id for differentiate session when doing matches

    Properties:
        url: /sessionhistory/v2/admin/namespaces/{namespace}/xray/tickets

        method: POST

        tags: ["XRay"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsXRayTicketObservabilityRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsXRayTicketObservabilityResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateXrayTicketObservability.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelCountryBlacklistRequest
from ..models import ModelCountryBlacklistResponse
from ..models import ModelCountryResponse
from ..models import RestErrorResponse

from ..operations.country import AdminAddCountryBlacklistV3
from ..operations.country import AdminGetCountryBlacklistV3
from ..operations.country import AdminGetCountryListV3
from ..operations.country import PublicGetCountryListV3


@same_doc_as(AdminAddCountryBlacklistV3)
def admin_add_country_blacklist_v3(
    body: ModelCountryBlacklistRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Country Blacklist (AdminAddCountryBlacklistV3)

    Updates the country blacklist for the namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/countries/blacklist

        method: POST

        tags: ["Country"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelCountryBlacklistRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminAddCountryBlacklistV3.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminAddCountryBlacklistV3)
async def admin_add_country_blacklist_v3_async(
    body: ModelCountryBlacklistRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Country Blacklist (AdminAddCountryBlacklistV3)

    Updates the country blacklist for the namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/countries/blacklist

        method: POST

        tags: ["Country"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelCountryBlacklistRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminAddCountryBlacklistV3.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetCountryBlacklistV3)
def admin_get_country_blacklist_v3(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Country Blacklist (AdminGetCountryBlacklistV3)

    Retrieves the country blacklist for the namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/countries/blacklist

        method: GET

        tags: ["Country"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelCountryBlacklistResponse (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetCountryBlacklistV3.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetCountryBlacklistV3)
async def admin_get_country_blacklist_v3_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Country Blacklist (AdminGetCountryBlacklistV3)

    Retrieves the country blacklist for the namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/countries/blacklist

        method: GET

        tags: ["Country"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelCountryBlacklistResponse (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetCountryBlacklistV3.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetCountryListV3)
def admin_get_country_list_v3(
    filter_blacklist: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Country List (AdminGetCountryListV3)

    Retrieves the list of countries for the namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/countries

        method: GET

        tags: ["Country"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        filter_blacklist: (filterBlacklist) OPTIONAL bool in query

    Responses:
        200: OK - List[ModelCountryResponse] (No Content)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetCountryListV3.create(
        filter_blacklist=filter_blacklist,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetCountryListV3)
async def admin_get_country_list_v3_async(
    filter_blacklist: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Country List (AdminGetCountryListV3)

    Retrieves the list of countries for the namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/countries

        method: GET

        tags: ["Country"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        filter_blacklist: (filterBlacklist) OPTIONAL bool in query

    Responses:
        200: OK - List[ModelCountryResponse] (No Content)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetCountryListV3.create(
        filter_blacklist=filter_blacklist,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetCountryListV3)
def public_get_country_list_v3(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Country List (PublicGetCountryListV3)

    Retrieves the list of countries, excluding those on the blacklist.

    Properties:
        url: /iam/v3/public/namespaces/{namespace}/countries

        method: GET

        tags: ["Country"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelCountryResponse] (OK)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicGetCountryListV3.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetCountryListV3)
async def public_get_country_list_v3_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Country List (PublicGetCountryListV3)

    Retrieves the list of countries, excluding those on the blacklist.

    Properties:
        url: /iam/v3/public/namespaces/{namespace}/countries

        method: GET

        tags: ["Country"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelCountryResponse] (OK)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicGetCountryListV3.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the Fleet Commander."""

__version__ = "1.46.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

# account
from .wrappers import account_get
from .wrappers import account_get_async
from .wrappers import admin_account_create
from .wrappers import admin_account_create_async
from .wrappers import admin_account_get
from .wrappers import admin_account_get_async
from .wrappers import admin_account_link
from .wrappers import admin_account_link_async
from .wrappers import admin_account_link_token_get
from .wrappers import admin_account_link_token_get_async

# ams_info
from .wrappers import info_regions
from .wrappers import info_regions_async
from .wrappers import info_supported_instances
from .wrappers import info_supported_instances_async
from .wrappers import upload_url_get
from .wrappers import upload_url_get_async

# ams_qo_s
from .wrappers import qo_s_regions_get
from .wrappers import qo_s_regions_get_async
from .wrappers import qo_s_regions_update
from .wrappers import qo_s_regions_update_async

# artifacts
from .wrappers import artifact_bulk_delete
from .wrappers import artifact_bulk_delete_async
from .wrappers import artifact_delete
from .wrappers import artifact_delete_async
from .wrappers import artifact_get
from .wrappers import artifact_get_async
from .wrappers import artifact_get_url
from .wrappers import artifact_get_url_async
from .wrappers import artifact_usage_get
from .wrappers import artifact_usage_get_async
from .wrappers import fleet_artifact_sampling_rules_get
from .wrappers import fleet_artifact_sampling_rules_get_async
from .wrappers import fleet_artifact_sampling_rules_set
from .wrappers import fleet_artifact_sampling_rules_set_async

# auth
from .wrappers import auth_check
from .wrappers import auth_check_async

# development
from .wrappers import development_server_configuration_create
from .wrappers import development_server_configuration_create_async
from .wrappers import development_server_configuration_delete
from .wrappers import development_server_configuration_delete_async
from .wrappers import development_server_configuration_get
from .wrappers import development_server_configuration_get_async
from .wrappers import development_server_configuration_list
from .wrappers import development_server_configuration_list_async
from .wrappers import development_server_configuration_patch
from .wrappers import development_server_configuration_patch_async

# fleet_commander
from .wrappers import basic_health_check
from .wrappers import basic_health_check_async
from .wrappers import portal_health_check
from .wrappers import portal_health_check_async
from .wrappers import version_info
from .wrappers import version_info_async

# fleets
from .wrappers import bulk_fleet_delete
from .wrappers import bulk_fleet_delete_async
from .wrappers import fleet_claim_by_id
from .wrappers import fleet_claim_by_id_async
from .wrappers import fleet_claim_by_keys
from .wrappers import fleet_claim_by_keys_async
from .wrappers import fleet_create
from .wrappers import fleet_create_async
from .wrappers import fleet_delete
from .wrappers import fleet_delete_async
from .wrappers import fleet_get
from .wrappers import fleet_get_async
from .wrappers import fleet_list
from .wrappers import fleet_list_async
from .wrappers import fleet_servers
from .wrappers import fleet_servers_async
from .wrappers import fleet_update
from .wrappers import fleet_update_async

# images
from .wrappers import image_get
from .wrappers import image_get_async
from .wrappers import image_list
from .wrappers import image_list_async
from .wrappers import image_mark_for_deletion
from .wrappers import image_mark_for_deletion_async
from .wrappers import image_patch
from .wrappers import image_patch_async
from .wrappers import image_unmark_for_deletion
from .wrappers import image_unmark_for_deletion_async
from .wrappers import images_storage
from .wrappers import images_storage_async

# servers
from .wrappers import fleet_server_connection_info
from .wrappers import fleet_server_connection_info_async
from .wrappers import fleet_server_history
from .wrappers import fleet_server_history_async
from .wrappers import fleet_server_info
from .wrappers import fleet_server_info_async
from .wrappers import server_history
from .wrappers import server_history_async

# watchdogs
from .wrappers import local_watchdog_connect
from .wrappers import local_watchdog_connect_async
from .wrappers import watchdog_connect
from .wrappers import watchdog_connect_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Inventory Service Early Access

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelsCreateInventoryConfigurationReq(Model):
    """Apimodels create inventory configuration req (apimodels.CreateInventoryConfigurationReq)

    Properties:
        code: (code) REQUIRED str

        initial_max_slots: (initialMaxSlots) REQUIRED int

        max_instances_per_user: (maxInstancesPerUser) REQUIRED int

        max_upgrade_slots: (maxUpgradeSlots) REQUIRED int

        description: (description) OPTIONAL str

        name: (name) OPTIONAL str
    """

    # region fields

    code: str  # REQUIRED
    initial_max_slots: int  # REQUIRED
    max_instances_per_user: int  # REQUIRED
    max_upgrade_slots: int  # REQUIRED
    description: str  # OPTIONAL
    name: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_code(self, value: str) -> ApimodelsCreateInventoryConfigurationReq:
        self.code = value
        return self

    def with_initial_max_slots(
        self, value: int
    ) -> ApimodelsCreateInventoryConfigurationReq:
        self.initial_max_slots = value
        return self

    def with_max_instances_per_user(
        self, value: int
    ) -> ApimodelsCreateInventoryConfigurationReq:
        self.max_instances_per_user = value
        return self

    def with_max_upgrade_slots(
        self, value: int
    ) -> ApimodelsCreateInventoryConfigurationReq:
        self.max_upgrade_slots = value
        return self

    def with_description(self, value: str) -> ApimodelsCreateInventoryConfigurationReq:
        self.description = value
        return self

    def with_name(self, value: str) -> ApimodelsCreateInventoryConfigurationReq:
        self.name = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "code"):
            result["code"] = str(self.code)
        elif include_empty:
            result["code"] = ""
        if hasattr(self, "initial_max_slots"):
            result["initialMaxSlots"] = int(self.initial_max_slots)
        elif include_empty:
            result["initialMaxSlots"] = 0
        if hasattr(self, "max_instances_per_user"):
            result["maxInstancesPerUser"] = int(self.max_instances_per_user)
        elif include_empty:
            result["maxInstancesPerUser"] = 0
        if hasattr(self, "max_upgrade_slots"):
            result["maxUpgradeSlots"] = int(self.max_upgrade_slots)
        elif include_empty:
            result["maxUpgradeSlots"] = 0
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        code: str,
        initial_max_slots: int,
        max_instances_per_user: int,
        max_upgrade_slots: int,
        description: Optional[str] = None,
        name: Optional[str] = None,
        **kwargs,
    ) -> ApimodelsCreateInventoryConfigurationReq:
        instance = cls()
        instance.code = code
        instance.initial_max_slots = initial_max_slots
        instance.max_instances_per_user = max_instances_per_user
        instance.max_upgrade_slots = max_upgrade_slots
        if description is not None:
            instance.description = description
        if name is not None:
            instance.name = name
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsCreateInventoryConfigurationReq:
        instance = cls()
        if not dict_:
            return instance
        if "code" in dict_ and dict_["code"] is not None:
            instance.code = str(dict_["code"])
        elif include_empty:
            instance.code = ""
        if "initialMaxSlots" in dict_ and dict_["initialMaxSlots"] is not None:
            instance.initial_max_slots = int(dict_["initialMaxSlots"])
        elif include_empty:
            instance.initial_max_slots = 0
        if "maxInstancesPerUser" in dict_ and dict_["maxInstancesPerUser"] is not None:
            instance.max_instances_per_user = int(dict_["maxInstancesPerUser"])
        elif include_empty:
            instance.max_instances_per_user = 0
        if "maxUpgradeSlots" in dict_ and dict_["maxUpgradeSlots"] is not None:
            instance.max_upgrade_slots = int(dict_["maxUpgradeSlots"])
        elif include_empty:
            instance.max_upgrade_slots = 0
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsCreateInventoryConfigurationReq]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsCreateInventoryConfigurationReq]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsCreateInventoryConfigurationReq,
        List[ApimodelsCreateInventoryConfigurationReq],
        Dict[Any, ApimodelsCreateInventoryConfigurationReq],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "code": "code",
            "initialMaxSlots": "initial_max_slots",
            "maxInstancesPerUser": "max_instances_per_user",
            "maxUpgradeSlots": "max_upgrade_slots",
            "description": "description",
            "name": "name",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "code": True,
            "initialMaxSlots": True,
            "maxInstancesPerUser": True,
            "maxUpgradeSlots": True,
            "description": False,
            "name": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Iam Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import OauthmodelLoginQueueTicketResponse
from ...models import OauthmodelTokenResponseV3
from ...models import RestErrorResponse


class GenerateTokenByNewHeadlessAccountV4(Operation):
    """Create headless account and response token (GenerateTokenByNewHeadlessAccountV4)

    Creates headless account after 3rd platform authenticated, and returns token.
    The 'linkingToken' in request body is received from "/platforms/{platformId}/token" when 3rd platform account is not linked to any account yet and createHeadless param is set to false.

    ## Login Queue
    When the Login Queue is enabled and at capacity, this API returns a 202 Accepted response, with the queue ticket included in the response body.

    Properties:
        url: /iam/v4/oauth/headless/token

        method: POST

        tags: ["OAuth2.0 V4"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        additional_data: (additionalData) OPTIONAL str in form_data

        extend_exp: (extend_exp) OPTIONAL bool in form_data

        linking_token: (linkingToken) REQUIRED str in form_data

    Responses:
        200: OK - OauthmodelTokenResponseV3 (Succeed to create headless account and response token info.)

        202: Accepted - OauthmodelLoginQueueTicketResponse (Succeed to create headless account and response token info.)

        400: Bad Request - RestErrorResponse (Invalid platform linking token or clientID not match.)

        401: Unauthorized - RestErrorResponse (10213: country is blocked)

        404: Not Found - RestErrorResponse (Platform linking token not found.)
    """

    # region fields

    _url: str = "/iam/v4/oauth/headless/token"
    _method: str = "POST"
    _consumes: List[str] = ["application/x-www-form-urlencoded"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    additional_data: str  # OPTIONAL in [form_data]
    extend_exp: bool  # OPTIONAL in [form_data]
    linking_token: str  # REQUIRED in [form_data]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "form_data": self.get_form_data_params(),
        }

    def get_form_data_params(self) -> dict:
        result = {}
        if hasattr(self, "additional_data"):
            result["additionalData"] = self.additional_data
        if hasattr(self, "extend_exp"):
            result["extend_exp"] = self.extend_exp
        if hasattr(self, "linking_token"):
            result["linkingToken"] = self.linking_token
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_additional_data(self, value: str) -> GenerateTokenByNewHeadlessAccountV4:
        self.additional_data = value
        return self

    def with_extend_exp(self, value: bool) -> GenerateTokenByNewHeadlessAccountV4:
        self.extend_exp = value
        return self

    def with_linking_token(self, value: str) -> GenerateTokenByNewHeadlessAccountV4:
        self.linking_token = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "additional_data") and self.additional_data:
            result["additionalData"] = str(self.additional_data)
        elif include_empty:
            result["additionalData"] = ""
        if hasattr(self, "extend_exp") and self.extend_exp:
            result["extend_exp"] = bool(self.extend_exp)
        elif include_empty:
            result["extend_exp"] = False
        if hasattr(self, "linking_token") and self.linking_token:
            result["linkingToken"] = str(self.linking_token)
        elif include_empty:
            result["linkingToken"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, OauthmodelLoginQueueTicketResponse, OauthmodelTokenResponseV3],
        Union[None, HttpResponse, RestErrorResponse],
    ]:
        """Parse the given response.

        200: OK - OauthmodelTokenResponseV3 (Succeed to create headless account and response token info.)

        202: Accepted - OauthmodelLoginQueueTicketResponse (Succeed to create headless account and response token info.)

        400: Bad Request - RestErrorResponse (Invalid platform linking token or clientID not match.)

        401: Unauthorized - RestErrorResponse (10213: country is blocked)

        404: Not Found - RestErrorResponse (Platform linking token not found.)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return OauthmodelTokenResponseV3.create_from_dict(content), None
        if code == 202:
            return OauthmodelLoginQueueTicketResponse.create_from_dict(content), None
        if code == 400:
            return None, RestErrorResponse.create_from_dict(content)
        if code == 401:
            return None, RestErrorResponse.create_from_dict(content)
        if code == 404:
            return None, RestErrorResponse.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        linking_token: str,
        additional_data: Optional[str] = None,
        extend_exp: Optional[bool] = None,
        **kwargs,
    ) -> GenerateTokenByNewHeadlessAccountV4:
        instance = cls()
        instance.linking_token = linking_token
        if additional_data is not None:
            instance.additional_data = additional_data
        if extend_exp is not None:
            instance.extend_exp = extend_exp
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> GenerateTokenByNewHeadlessAccountV4:
        instance = cls()
        if "additionalData" in dict_ and dict_["additionalData"] is not None:
            instance.additional_data = str(dict_["additionalData"])
        elif include_empty:
            instance.additional_data = ""
        if "extend_exp" in dict_ and dict_["extend_exp"] is not None:
            instance.extend_exp = bool(dict_["extend_exp"])
        elif include_empty:
            instance.extend_exp = False
        if "linkingToken" in dict_ and dict_["linkingToken"] is not None:
            instance.linking_token = str(dict_["linkingToken"])
        elif include_empty:
            instance.linking_token = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "additionalData": "additional_data",
            "extend_exp": "extend_exp",
            "linkingToken": "linking_token",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "additionalData": False,
            "extend_exp": False,
            "linkingToken": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApiQoSEndpointResponse
from ..models import ApiUpdateServerRequest
from ..models import ResponseErrorResponse

from ..operations.ams_qo_s import QoSRegionsGet
from ..operations.ams_qo_s import QoSRegionsUpdate


@same_doc_as(QoSRegionsGet)
def qo_s_regions_get(
    status: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get a list of available AMS QoS regions (QoSRegionsGet)

    ```
    Required Permission: ADMIN:NAMESPACE:{namespace}:QOS:SERVER [READ]

    This endpoint lists all QoS services available in all regions.

    This endpoint is intended to be called by game client to find out all available regions.
    After getting a list of QoS on each region, game client is expected to ping each one with UDP
    connection as described below:

    1. Make UDP connection to each QoS's IP:Port
    2. Send string "PING" after connection established
    3. Wait for string "PONG" response
    4. Note the request-response latency for each QoS in each region

    The game then can use ping latency information to either:
    1. Inform the player on these latencies and let player choose preferred region
    2. Send the latency list to Matchmaking Service so that player can be matched with other players
    in nearby regions

    Properties:
        url: /ams/v1/admin/namespaces/{namespace}/qos

        method: GET

        tags: ["AMS QoS"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        status: (status) OPTIONAL str in query

    Responses:
        200: OK - ApiQoSEndpointResponse (success)

        400: Bad Request - ResponseErrorResponse (bad request)

        401: Unauthorized - ResponseErrorResponse (no authorization provided)

        403: Forbidden - ResponseErrorResponse (insufficient permissions)

        500: Internal Server Error - ResponseErrorResponse (internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = QoSRegionsGet.create(
        status=status,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QoSRegionsGet)
async def qo_s_regions_get_async(
    status: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """get a list of available AMS QoS regions (QoSRegionsGet)

    ```
    Required Permission: ADMIN:NAMESPACE:{namespace}:QOS:SERVER [READ]

    This endpoint lists all QoS services available in all regions.

    This endpoint is intended to be called by game client to find out all available regions.
    After getting a list of QoS on each region, game client is expected to ping each one with UDP
    connection as described below:

    1. Make UDP connection to each QoS's IP:Port
    2. Send string "PING" after connection established
    3. Wait for string "PONG" response
    4. Note the request-response latency for each QoS in each region

    The game then can use ping latency information to either:
    1. Inform the player on these latencies and let player choose preferred region
    2. Send the latency list to Matchmaking Service so that player can be matched with other players
    in nearby regions

    Properties:
        url: /ams/v1/admin/namespaces/{namespace}/qos

        method: GET

        tags: ["AMS QoS"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        status: (status) OPTIONAL str in query

    Responses:
        200: OK - ApiQoSEndpointResponse (success)

        400: Bad Request - ResponseErrorResponse (bad request)

        401: Unauthorized - ResponseErrorResponse (no authorization provided)

        403: Forbidden - ResponseErrorResponse (insufficient permissions)

        500: Internal Server Error - ResponseErrorResponse (internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = QoSRegionsGet.create(
        status=status,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QoSRegionsUpdate)
def qo_s_regions_update(
    body: ApiUpdateServerRequest,
    region: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """update the status of a QoS region (QoSRegionsUpdate)

    ```
    Required Permission: ADMIN:NAMESPACE:{namespace}:QOS:SERVER [UPDATE]

    This endpoint updates the registered QoS service's configurable configuration.

    Properties:
        url: /ams/v1/admin/namespaces/{namespace}/qos/{region}

        method: PATCH

        tags: ["AMS QoS"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiUpdateServerRequest in body

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

    Responses:
        204: No Content - (record updated)

        400: Bad Request - ResponseErrorResponse (malformed request)

        401: Unauthorized - ResponseErrorResponse (no authorization provided)

        403: Forbidden - ResponseErrorResponse (insufficient permissions)

        404: Not Found - ResponseErrorResponse (regionial QoS record not found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = QoSRegionsUpdate.create(
        body=body,
        region=region,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QoSRegionsUpdate)
async def qo_s_regions_update_async(
    body: ApiUpdateServerRequest,
    region: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """update the status of a QoS region (QoSRegionsUpdate)

    ```
    Required Permission: ADMIN:NAMESPACE:{namespace}:QOS:SERVER [UPDATE]

    This endpoint updates the registered QoS service's configurable configuration.

    Properties:
        url: /ams/v1/admin/namespaces/{namespace}/qos/{region}

        method: PATCH

        tags: ["AMS QoS"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiUpdateServerRequest in body

        namespace: (namespace) REQUIRED str in path

        region: (region) REQUIRED str in path

    Responses:
        204: No Content - (record updated)

        400: Bad Request - ResponseErrorResponse (malformed request)

        401: Unauthorized - ResponseErrorResponse (no authorization provided)

        403: Forbidden - ResponseErrorResponse (insufficient permissions)

        404: Not Found - ResponseErrorResponse (regionial QoS record not found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = QoSRegionsUpdate.create(
        body=body,
        region=region,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.fulfillment_v2_result import FulfillmentV2Result
from ..models.revoke_fulfillment_v2_result import RevokeFulfillmentV2Result


class ItemIdentityTypeEnum(StrEnum):
    ITEM_ID = "ITEM_ID"
    ITEM_SKU = "ITEM_SKU"


class PlatformEnum(StrEnum):
    STEAM = "STEAM"


class StatusEnum(StrEnum):
    FULFILLED = "FULFILLED"
    VERIFIED = "VERIFIED"


class IAPOrderLineItemInfo(Model):
    """IAP order line item info (IAPOrderLineItemInfo)

    Properties:
        amount: (amount) OPTIONAL int

        iap_order_no: (iapOrderNo) OPTIONAL str

        id_: (id) OPTIONAL str

        item_identity: (itemIdentity) OPTIONAL str

        item_identity_type: (itemIdentityType) OPTIONAL Union[str, ItemIdentityTypeEnum]

        last_fulfillment_v2_result: (lastFulfillmentV2Result) OPTIONAL FulfillmentV2Result

        last_revoke_fulfillment_v2_result: (lastRevokeFulfillmentV2Result) OPTIONAL RevokeFulfillmentV2Result

        line_item_id: (lineItemId) OPTIONAL str

        namespace: (namespace) OPTIONAL str

        platform: (platform) OPTIONAL Union[str, PlatformEnum]

        qty: (qty) OPTIONAL int

        sandbox: (sandbox) OPTIONAL bool

        status: (status) OPTIONAL Union[str, StatusEnum]

        status_reason: (statusReason) OPTIONAL str

        third_party_item_id: (thirdPartyItemId) OPTIONAL str

        third_party_order_id: (thirdPartyOrderId) OPTIONAL str

        user_id: (userId) OPTIONAL str

        vat: (vat) OPTIONAL int
    """

    # region fields

    amount: int  # OPTIONAL
    iap_order_no: str  # OPTIONAL
    id_: str  # OPTIONAL
    item_identity: str  # OPTIONAL
    item_identity_type: Union[str, ItemIdentityTypeEnum]  # OPTIONAL
    last_fulfillment_v2_result: FulfillmentV2Result  # OPTIONAL
    last_revoke_fulfillment_v2_result: RevokeFulfillmentV2Result  # OPTIONAL
    line_item_id: str  # OPTIONAL
    namespace: str  # OPTIONAL
    platform: Union[str, PlatformEnum]  # OPTIONAL
    qty: int  # OPTIONAL
    sandbox: bool  # OPTIONAL
    status: Union[str, StatusEnum]  # OPTIONAL
    status_reason: str  # OPTIONAL
    third_party_item_id: str  # OPTIONAL
    third_party_order_id: str  # OPTIONAL
    user_id: str  # OPTIONAL
    vat: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_amount(self, value: int) -> IAPOrderLineItemInfo:
        self.amount = value
        return self

    def with_iap_order_no(self, value: str) -> IAPOrderLineItemInfo:
        self.iap_order_no = value
        return self

    def with_id(self, value: str) -> IAPOrderLineItemInfo:
        self.id_ = value
        return self

    def with_item_identity(self, value: str) -> IAPOrderLineItemInfo:
        self.item_identity = value
        return self

    def with_item_identity_type(
        self, value: Union[str, ItemIdentityTypeEnum]
    ) -> IAPOrderLineItemInfo:
        self.item_identity_type = value
        return self

    def with_last_fulfillment_v2_result(
        self, value: FulfillmentV2Result
    ) -> IAPOrderLineItemInfo:
        self.last_fulfillment_v2_result = value
        return self

    def with_last_revoke_fulfillment_v2_result(
        self, value: RevokeFulfillmentV2Result
    ) -> IAPOrderLineItemInfo:
        self.last_revoke_fulfillment_v2_result = value
        return self

    def with_line_item_id(self, value: str) -> IAPOrderLineItemInfo:
        self.line_item_id = value
        return self

    def with_namespace(self, value: str) -> IAPOrderLineItemInfo:
        self.namespace = value
        return self

    def with_platform(self, value: Union[str, PlatformEnum]) -> IAPOrderLineItemInfo:
        self.platform = value
        return self

    def with_qty(self, value: int) -> IAPOrderLineItemInfo:
        self.qty = value
        return self

    def with_sandbox(self, value: bool) -> IAPOrderLineItemInfo:
        self.sandbox = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> IAPOrderLineItemInfo:
        self.status = value
        return self

    def with_status_reason(self, value: str) -> IAPOrderLineItemInfo:
        self.status_reason = value
        return self

    def with_third_party_item_id(self, value: str) -> IAPOrderLineItemInfo:
        self.third_party_item_id = value
        return self

    def with_third_party_order_id(self, value: str) -> IAPOrderLineItemInfo:
        self.third_party_order_id = value
        return self

    def with_user_id(self, value: str) -> IAPOrderLineItemInfo:
        self.user_id = value
        return self

    def with_vat(self, value: int) -> IAPOrderLineItemInfo:
        self.vat = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "amount"):
            result["amount"] = int(self.amount)
        elif include_empty:
            result["amount"] = 0
        if hasattr(self, "iap_order_no"):
            result["iapOrderNo"] = str(self.iap_order_no)
        elif include_empty:
            result["iapOrderNo"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "item_identity"):
            result["itemIdentity"] = str(self.item_identity)
        elif include_empty:
            result["itemIdentity"] = ""
        if hasattr(self, "item_identity_type"):
            result["itemIdentityType"] = str(self.item_identity_type)
        elif include_empty:
            result["itemIdentityType"] = Union[str, ItemIdentityTypeEnum]()
        if hasattr(self, "last_fulfillment_v2_result"):
            result["lastFulfillmentV2Result"] = self.last_fulfillment_v2_result.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["lastFulfillmentV2Result"] = FulfillmentV2Result()
        if hasattr(self, "last_revoke_fulfillment_v2_result"):
            result[
                "lastRevokeFulfillmentV2Result"
            ] = self.last_revoke_fulfillment_v2_result.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["lastRevokeFulfillmentV2Result"] = RevokeFulfillmentV2Result()
        if hasattr(self, "line_item_id"):
            result["lineItemId"] = str(self.line_item_id)
        elif include_empty:
            result["lineItemId"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "platform"):
            result["platform"] = str(self.platform)
        elif include_empty:
            result["platform"] = Union[str, PlatformEnum]()
        if hasattr(self, "qty"):
            result["qty"] = int(self.qty)
        elif include_empty:
            result["qty"] = 0
        if hasattr(self, "sandbox"):
            result["sandbox"] = bool(self.sandbox)
        elif include_empty:
            result["sandbox"] = False
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "status_reason"):
            result["statusReason"] = str(self.status_reason)
        elif include_empty:
            result["statusReason"] = ""
        if hasattr(self, "third_party_item_id"):
            result["thirdPartyItemId"] = str(self.third_party_item_id)
        elif include_empty:
            result["thirdPartyItemId"] = ""
        if hasattr(self, "third_party_order_id"):
            result["thirdPartyOrderId"] = str(self.third_party_order_id)
        elif include_empty:
            result["thirdPartyOrderId"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "vat"):
            result["vat"] = int(self.vat)
        elif include_empty:
            result["vat"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        amount: Optional[int] = None,
        iap_order_no: Optional[str] = None,
        id_: Optional[str] = None,
        item_identity: Optional[str] = None,
        item_identity_type: Optional[Union[str, ItemIdentityTypeEnum]] = None,
        last_fulfillment_v2_result: Optional[FulfillmentV2Result] = None,
        last_revoke_fulfillment_v2_result: Optional[RevokeFulfillmentV2Result] = None,
        line_item_id: Optional[str] = None,
        namespace: Optional[str] = None,
        platform: Optional[Union[str, PlatformEnum]] = None,
        qty: Optional[int] = None,
        sandbox: Optional[bool] = None,
        status: Optional[Union[str, StatusEnum]] = None,
        status_reason: Optional[str] = None,
        third_party_item_id: Optional[str] = None,
        third_party_order_id: Optional[str] = None,
        user_id: Optional[str] = None,
        vat: Optional[int] = None,
        **kwargs,
    ) -> IAPOrderLineItemInfo:
        instance = cls()
        if amount is not None:
            instance.amount = amount
        if iap_order_no is not None:
            instance.iap_order_no = iap_order_no
        if id_ is not None:
            instance.id_ = id_
        if item_identity is not None:
            instance.item_identity = item_identity
        if item_identity_type is not None:
            instance.item_identity_type = item_identity_type
        if last_fulfillment_v2_result is not None:
            instance.last_fulfillment_v2_result = last_fulfillment_v2_result
        if last_revoke_fulfillment_v2_result is not None:
            instance.last_revoke_fulfillment_v2_result = (
                last_revoke_fulfillment_v2_result
            )
        if line_item_id is not None:
            instance.line_item_id = line_item_id
        if namespace is not None:
            instance.namespace = namespace
        if platform is not None:
            instance.platform = platform
        if qty is not None:
            instance.qty = qty
        if sandbox is not None:
            instance.sandbox = sandbox
        if status is not None:
            instance.status = status
        if status_reason is not None:
            instance.status_reason = status_reason
        if third_party_item_id is not None:
            instance.third_party_item_id = third_party_item_id
        if third_party_order_id is not None:
            instance.third_party_order_id = third_party_order_id
        if user_id is not None:
            instance.user_id = user_id
        if vat is not None:
            instance.vat = vat
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> IAPOrderLineItemInfo:
        instance = cls()
        if not dict_:
            return instance
        if "amount" in dict_ and dict_["amount"] is not None:
            instance.amount = int(dict_["amount"])
        elif include_empty:
            instance.amount = 0
        if "iapOrderNo" in dict_ and dict_["iapOrderNo"] is not None:
            instance.iap_order_no = str(dict_["iapOrderNo"])
        elif include_empty:
            instance.iap_order_no = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "itemIdentity" in dict_ and dict_["itemIdentity"] is not None:
            instance.item_identity = str(dict_["itemIdentity"])
        elif include_empty:
            instance.item_identity = ""
        if "itemIdentityType" in dict_ and dict_["itemIdentityType"] is not None:
            instance.item_identity_type = str(dict_["itemIdentityType"])
        elif include_empty:
            instance.item_identity_type = Union[str, ItemIdentityTypeEnum]()
        if (
            "lastFulfillmentV2Result" in dict_
            and dict_["lastFulfillmentV2Result"] is not None
        ):
            instance.last_fulfillment_v2_result = FulfillmentV2Result.create_from_dict(
                dict_["lastFulfillmentV2Result"], include_empty=include_empty
            )
        elif include_empty:
            instance.last_fulfillment_v2_result = FulfillmentV2Result()
        if (
            "lastRevokeFulfillmentV2Result" in dict_
            and dict_["lastRevokeFulfillmentV2Result"] is not None
        ):
            instance.last_revoke_fulfillment_v2_result = (
                RevokeFulfillmentV2Result.create_from_dict(
                    dict_["lastRevokeFulfillmentV2Result"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.last_revoke_fulfillment_v2_result = RevokeFulfillmentV2Result()
        if "lineItemId" in dict_ and dict_["lineItemId"] is not None:
            instance.line_item_id = str(dict_["lineItemId"])
        elif include_empty:
            instance.line_item_id = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "platform" in dict_ and dict_["platform"] is not None:
            instance.platform = str(dict_["platform"])
        elif include_empty:
            instance.platform = Union[str, PlatformEnum]()
        if "qty" in dict_ and dict_["qty"] is not None:
            instance.qty = int(dict_["qty"])
        elif include_empty:
            instance.qty = 0
        if "sandbox" in dict_ and dict_["sandbox"] is not None:
            instance.sandbox = bool(dict_["sandbox"])
        elif include_empty:
            instance.sandbox = False
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "statusReason" in dict_ and dict_["statusReason"] is not None:
            instance.status_reason = str(dict_["statusReason"])
        elif include_empty:
            instance.status_reason = ""
        if "thirdPartyItemId" in dict_ and dict_["thirdPartyItemId"] is not None:
            instance.third_party_item_id = str(dict_["thirdPartyItemId"])
        elif include_empty:
            instance.third_party_item_id = ""
        if "thirdPartyOrderId" in dict_ and dict_["thirdPartyOrderId"] is not None:
            instance.third_party_order_id = str(dict_["thirdPartyOrderId"])
        elif include_empty:
            instance.third_party_order_id = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "vat" in dict_ and dict_["vat"] is not None:
            instance.vat = int(dict_["vat"])
        elif include_empty:
            instance.vat = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, IAPOrderLineItemInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[IAPOrderLineItemInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        IAPOrderLineItemInfo,
        List[IAPOrderLineItemInfo],
        Dict[Any, IAPOrderLineItemInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "amount": "amount",
            "iapOrderNo": "iap_order_no",
            "id": "id_",
            "itemIdentity": "item_identity",
            "itemIdentityType": "item_identity_type",
            "lastFulfillmentV2Result": "last_fulfillment_v2_result",
            "lastRevokeFulfillmentV2Result": "last_revoke_fulfillment_v2_result",
            "lineItemId": "line_item_id",
            "namespace": "namespace",
            "platform": "platform",
            "qty": "qty",
            "sandbox": "sandbox",
            "status": "status",
            "statusReason": "status_reason",
            "thirdPartyItemId": "third_party_item_id",
            "thirdPartyOrderId": "third_party_order_id",
            "userId": "user_id",
            "vat": "vat",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "amount": False,
            "iapOrderNo": False,
            "id": False,
            "itemIdentity": False,
            "itemIdentityType": False,
            "lastFulfillmentV2Result": False,
            "lastRevokeFulfillmentV2Result": False,
            "lineItemId": False,
            "namespace": False,
            "platform": False,
            "qty": False,
            "sandbox": False,
            "status": False,
            "statusReason": False,
            "thirdPartyItemId": False,
            "thirdPartyOrderId": False,
            "userId": False,
            "vat": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "itemIdentityType": ["ITEM_ID", "ITEM_SKU"],
            "platform": ["STEAM"],
            "status": ["FULFILLED", "VERIFIED"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApimodelAppDebugInfoResponse
from ..models import ApimodelAppItem
from ..models import ApimodelUpdateDebugModeRequest
from ..models import ResponseErrorResponse

from ..operations.app_v4 import GetAppDebugInfoV4
from ..operations.app_v4 import UpdateAppDebugModeV4


@same_doc_as(GetAppDebugInfoV4)
def get_app_debug_info_v4(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get debug info for an extend app (GetAppDebugInfoV4)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [READ]`

    Returns the list of debug-ready pods and exposed services for the given extend app.

    A pod is included only when it is in `Running` phase and has an
    `extend-proxy-sidecar` container.

    Properties:
        url: /csm/v4/admin/namespaces/{namespace}/apps/{app}/debuginfo

        method: GET

        tags: ["App V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppDebugInfoResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetAppDebugInfoV4.create(
        app=app,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetAppDebugInfoV4)
async def get_app_debug_info_v4_async(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get debug info for an extend app (GetAppDebugInfoV4)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [READ]`

    Returns the list of debug-ready pods and exposed services for the given extend app.

    A pod is included only when it is in `Running` phase and has an
    `extend-proxy-sidecar` container.

    Properties:
        url: /csm/v4/admin/namespaces/{namespace}/apps/{app}/debuginfo

        method: GET

        tags: ["App V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppDebugInfoResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetAppDebugInfoV4.create(
        app=app,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateAppDebugModeV4)
def update_app_debug_mode_v4(
    app: str,
    body: ApimodelUpdateDebugModeRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Enable or disable remote debug mode for an extend app (UpdateAppDebugModeV4)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [UPDATE]`

    Enable or disable the remote debug sidecar container for the specified extend app.

    When enabled and the app is in `deployment-running` status, a new deployment will be
    triggered automatically to apply the debug container configuration.

    This operation is rejected with `409 Conflict` if the app is currently in a
    transitional state (app-creating, app-removing, app-stopping).

    Properties:
        url: /csm/v4/admin/namespaces/{namespace}/apps/{app}/debugmode

        method: PUT

        tags: ["App V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelUpdateDebugModeRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateAppDebugModeV4.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateAppDebugModeV4)
async def update_app_debug_mode_v4_async(
    app: str,
    body: ApimodelUpdateDebugModeRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Enable or disable remote debug mode for an extend app (UpdateAppDebugModeV4)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [UPDATE]`

    Enable or disable the remote debug sidecar container for the specified extend app.

    When enabled and the app is in `deployment-running` status, a new deployment will be
    triggered automatically to apply the debug container configuration.

    This operation is rejected with `409 Conflict` if the app is currently in a
    transitional state (app-creating, app-removing, app-stopping).

    Properties:
        url: /csm/v4/admin/namespaces/{namespace}/apps/{app}/debugmode

        method: PUT

        tags: ["App V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelUpdateDebugModeRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelAppItem (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateAppDebugModeV4.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2025 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from typing import Dict, Optional

from ...core import (
    SDK,
    AccelByteSDK,
    HttpResponse,
    create_basic_authentication,
)

from ...api import iam
from ...api.iam.operations import o_auth2_0 as oauth2

from ._login_const import LOGIN_TYPE_CLIENT


def login_client_internal(
    client_id: Optional[str] = None,
    client_secret: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    sdk: Optional[AccelByteSDK] = None,
    **kwargs,
):
    sdk = sdk or SDK

    if not client_id and client_secret:
        return None, HttpResponse.create_error(400, "Invalid client ID.")

    if not client_id:
        client_auth, error = sdk.get_client_auth()
        if error:
            return None, error
        client_id, config_client_secret = client_auth
        if not client_secret:
            client_secret = config_client_secret

    if not client_secret:
        sdk.logger.warning(
            "The use of a Public OAuth Client is highly discouraged for this use case."
        )

    x_additional_headers = x_additional_headers or {}
    x_additional_headers["Authorization"] = create_basic_authentication(
        username=client_id,
        password=client_secret,
    )

    token, error = iam.token_grant_v3(
        grant_type=oauth2.TokenGrantV3GrantTypeEnum.CLIENT_CREDENTIALS,
        x_additional_headers=x_additional_headers,
        sdk=sdk,
        **kwargs,
    )
    if error:
        return None, error

    assert token, "Token value missing!"

    _, error = sdk.set_token(token=token, type=LOGIN_TYPE_CLIENT)
    if error:
        return None, error

    return token, None


async def login_client_async_internal(
    client_id: Optional[str] = None,
    client_secret: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    sdk: Optional[AccelByteSDK] = None,
    **kwargs,
):
    sdk = sdk or SDK

    if not client_id and client_secret:
        return None, HttpResponse.create_error(400, "Invalid client ID.")

    if not client_id:
        client_auth, error = sdk.get_client_auth()
        if error:
            return None, error
        client_id, config_client_secret = client_auth
        if not client_secret:
            client_secret = config_client_secret

    if not client_secret:
        sdk.logger.warning(
            "The use of a Public OAuth Client is highly discouraged for this use case."
        )

    x_additional_headers = x_additional_headers or {}
    x_additional_headers["Authorization"] = create_basic_authentication(
        username=client_id,
        password=client_secret,
    )

    token, error = await iam.token_grant_v3_async(
        grant_type=oauth2.TokenGrantV3GrantTypeEnum.CLIENT_CREDENTIALS,
        x_additional_headers=x_additional_headers,
        sdk=sdk,
        **kwargs,
    )
    if error:
        return None, error

    assert token, "Token value missing!"

    _, error = sdk.set_token(token=token, type=LOGIN_TYPE_CLIENT)
    if error:
        return None, error

    return token, None


__all__ = [
    "login_client_internal",
    "login_client_async_internal",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]
